﻿/**
 * This module handles data caching including checking for updated versions and cache invalidation
 *
 * Author: Waleed Madanat
 * Version: 1.0
 * Created: 8 Oct, 2013
 * Updated: 11 Oct, 2013
 */
var DataCacheManager = (function () {
    /**
     * Request the app version from the server, and if there is a difference
     * resets the cached data and updates the build numbers
     */
    function _versionCheck() {
        var url = window.host + '/api/designer/appVersion.json/' + window.appId;
        var xmlhttp = new XMLHttpRequest();
        xmlhttp.open("GET", url, true);
        xmlhttp.send();

        xmlhttp.onreadystatechange = function () {
            if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
                var contentVersion = localStorage['contentVersion'];
                var jsonObject = JSON.parse(xmlhttp.responseText);

                if (contentVersion != jsonObject.LiveContentVersion) {
                    // Reset the cache
                    // Remove all data except for the coreVersion which is used for
                    // versioning the bundle
                    for (var i = 0; i < localStorage.length; i++) {
                        if (localStorage.key(i) != 'bundleVersion') {
                            localStorage.removeItem(localStorage.key(i));
                        }
                    }

                    localStorage.setItem('buildVersion', jsonObject.ServerBuildVersion);
                    localStorage.setItem('contentVersion', jsonObject.LiveContentVersion);
                }
            }
        }
    }

    /**
     * Cache new data, the key is composed from the widget Id and Type
     * 
     * @param widgetId The widget's Id
     * @param widgetType The widget's type
     * @param data The data to cache
     */
    function _put(widgetId, widgetType, data) {
        localStorage.setItem(widgetType + widgetId, data);
    }

    /**
     * Fetch cached data, the key is composed from the widget Id and Type.
     * This metho will invalidate the cache in case there was a version change.
     * 
     * @param widgetId The widget's Id
     * @param widgetType The widget's type
     */
    function _get(widgetId, widgetType) {
        // Do the version check off the main UI thread
        // Because we don't want to delay the user longer, we'll
        // have to rely on cached data for this request and hope
        // that if it was updated, we'll get the fresh copy on the next request
        setTimeout(_versionCheck, 10);
        return localStorage.getItem(widgetType + widgetId);
    }

    return {
        put: _put,
        get: _get
    }
})();

if (typeof define === "function" && define.amd) {
    define("DataCacheManager", [], function () { return DataCacheManager; });
}