var dataFetcherTool = {
    appContentUrl: '/designer/appContent.json/[appId]',
    widgetUrl: '/designer/appContent/[appId]/[widget]widget.json/[widgetId]',
    appearanceUrl: '/designer/appContent/[appId]/appearance.json/[appearanceId]',
    version: 'df1.0',
    isProd: function () { return typeof (ENV) != "undefined" && ENV == "prod"; },
    isMobile: function () { return typeof (device) != "undefined" && device.platform && (device.platform.toLowerCase() == 'android' || device.platform.toLowerCase() == 'ios'); },
    useCDN: function () { return this.isProd() && this.isMobile(); },
    versionizeURL: function (page) {
        return this.jsUrl + page + (page.indexOf('?') > 0 ? '&' : '?') + 'cdnv=' + this.version;
    },
    init: function () {
        if (this.useCDN()) {
            //this.widgetUrl = 'http://cdn.myapp.kaleoapps.com' + this.widgetUrl;
            //this.appearanceUrl = 'http://cdn.myapp.kaleoapps.com' + this.appearanceUrl;
        }
    }
};

dataFetcherTool.init();



/**
 * This module is responsible for template data fetching from the API server,
 * it is also responsible for binding the data to the view.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 * Created: 28 Aug, 2013
 * Updated: 5 Sep, 2013
 */
define('DataFetcher',
['RestClient', 'jquery', 'knockout', 'komapping', 'LocalModel', 'TemplatesHelper'/*, 'DataCacheManager'*/],
function (rest, $, ko, kom, model, thelper/*, cacheMan*/) {
    // Map kom to ko.mapping, if not mapped already
    if (ko.mapping === undefined) {
        ko.mapping = kom;
    }

   /**
    * Fetches the app data, caches the app data model and
    * invokes a callback once done passing in the appId.
    *
    * @param callback The callback to invoke with the appId
    * @param showAlert In case the app wasn't published, an alert should show
    */
    var _getAppContent = function (callback, showAlert) {
        rest.get({
            serviceUrl: '/designer/app',
            Id: appId,
            success: function (data) {
                // Set the app data in the LocalModel module
                model.appModel(data);

                var appId = data.LiveContentId;

                // If preview mode has been set, fetch draft data instead of live ones
                if (window.previewMode || window.designMode) {
                    appId = data.DraftContentId;
                }

                if (appId) {
                    if (callback) {
                        callback(appId);
                    }
                } else {
                    document.body.style.background = '#151515';
                    //document.getElementById('loading').setAttribute("src", window.imgDownloadUrl + "/logo_loadingscreen.png");
                    //$('#loading').show();

                    if (showAlert) {
                        navigator.notification.confirm("Your application doesn\'t seem to have been published, there is nothing available to show here!\nWould you like to reload?",
                            function (buttonIndex) {
                                if (buttonIndex == 1) {
                                    window.location = './index.html';
                                } else {
                                    navigator.app.exitApp();
                                }
                            },
                            "No Data");
                    }
                }
            }
        });
    };

    /**
     * Get data for a certain widget from the API server
     *
     * @param widgetId The id of the widget to fetch data for
     * @param widgetType The type of widget to fetch data for
     * @param callback
     */
    var _get = function (widgetId, widgetType, callback) {
        _getAppContent(function (appId) {
            var requestURL;

            if (widgetType == thelper.TEMPLATE_KEYS.home) {
                requestURL = dataFetcherTool.appContentUrl.replace('[appId]', appId);
            } else {
                requestURL = dataFetcherTool.widgetUrl.replace('[appId]', appId).replace('[widget]', widgetType).replace('[widgetId]', widgetId)
            }

            // Fetch the cached data
            var apiData = undefined;
            // XXX: Caching is now disabled
            //if (!window.previewMode && !window.designMode) {
            //apiData = cacheMan.get(widgetId, widgetType);
            //}

            // No cached data, request from server
            if (apiData == undefined || apiData == null || apiData.replace(/\s+/g, '').length <= 0) {
                rest.get({
                    serviceUrl: requestURL,
                    success: function (xdata, textStatus, jqXHR) {
                        var content = JSON.stringify(xdata);

                        // Add the data to the cache, unless there is no data
                        if (content) {
                            // XXX: Disable caching
                            //cacheMan.put(widgetId, widgetType, content);

                            if (callback) {
                                callback(content);
                            }
                        } else {
                            navigator.notification.alert("Unable to data for chosen widget", null, "No Data", "OK");
                        }
                    }
                });
            } else {
                if (callback) {
                    callback(apiData);
                }
            }
        }, true);
    };

    /**
     * Get data for a certain widget from the API server
     *
     * @param appearanceId
     * @param callback
     */
    var _getAppearance = function (appearanceId, callback) {
        _getAppContent(function (appId) {
            var requestUrl = dataFetcherTool.appearanceUrl
                .replace('[appId]', appId)
                .replace('[appearanceId]', appearanceId);

            // No cached data, request from server
            rest.get({
                serviceUrl: requestUrl,
                success: function (xdata, textStatus, jqXHR) {
                    if (callback) {
                        model.appearanceModel(xdata);
                        callback(xdata);
                    }
                }
            });
        });
    };

    return {
        //getAppData: _getAppData,
        get: _get,
        getAppearance: _getAppearance
    }
});