﻿/**
 * This module enables the design related features required for authoring apps.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 * Created: 28 Aug, 2013
 * Updated: 4 Sep, 2013
 */
define('DesignMode',
['LocalModel', 'knockout', 'jquery', 'utility', 'kcomm', 'WidgetManager', 'TemplatesHelper', 'DataFetcher'],
function (model, ko, $, utility, kcomm, widman, thelper, dfetcher) {
    // An array of all of the home page widgets
    // useful in setting the current widget in case a new one came in
    var widgetIds = [];

    // A counter of the number of received app messages
    var appMessageCounter = 0;

    // Cached reference to document
    var $document = $(document);

    // A toggle to disable processing of app messages only once
    // The handler should flip it back
    var ignoreNextAppMessageToggle = false;

    // Register future handler for the new widget button on the emulator
    // Make sure to un-register any previous handler to avoid redundancy
    $document.off("click", ".homeIcon[data-type=new_widget]");
    $document.on("click", ".homeIcon[data-type=new_widget]",
    function (event) {
        // Reset the currently selected icon
        model.get(thelper.TEMPLATE_KEYS.home).currentWidget = 0;

        // Show the widget selector page on the content editor
        var message = '{"Instruction":{"Action":"New"}}';
        kcomm.sendMessage(message, parent.window);
    });

    /**
     * A click handler for widgets, it will send a postMessage to
     * the content editor
     */
    var _widgetClicked = function (id, type) {
        ignoreNextAppMessageToggle = true;
        var message = '{"Instruction":{"Action":"' + type + '", "Data":{"Id":"' + id + '"}}}';

        // Inform the content editor
        kcomm.sendMessage(message, parent.window);

        // Open the window after a delay
        // the content editor might send a message
        // as a reply to the message sent on the 
        // previous statement, wait until we are sure
        // we got it, then open the selected widget
        if (id && type) {
            widman.show(id, type.toLowerCase());
        }
    };

    /**
     * Removes the theme mock from the emulator if any.
     */
    var _removeThemeMock = function () {
        // In case use had updated the theme make sure to remove the div
        $themeMock = $("#themeMock");

        if ($themeMock.length > 0) {
            $themeMock.fadeOut(500, function () {
                $themeMock.remove();
            });
        }
    }

    /**
     * Re-applies the theme to the application.
     */
    var _refreshTheme = function () {
        dfetcher.get(0, thelper.TEMPLATE_KEYS.home, function (content) {
            var jsonObject = JSON.parse(content);
            dfetcher.getAppearance(jsonObject.AppearanceId, function (appearance) {
                // When done fadeing to black
                //
                // Re-apply the new theme
                // The id should not contain '/' or '.'
                var path = '/phone/css/defaultTheme.css'
                var id = path.replace(/\//g, "_").replace(/\./g, "_");

                // Remove the current style tag
                $("#" + id).remove();

                // Create a new one
                thelper.addStyleTag(id);
                var $style = $("#" + id);

                // Load the new style
                thelper.getResourceInText(path, function (data) {
                    data = data.replace(/_var_url/gi, window.host);

                    if (data.indexOf("@Theme_") >= 0) {
                        data = thelper.applyTheme(data, appearance);
                    }

                    $style.append(data);
                });
            });
        });
    }

    // ----------------------------------------------------------------------
    // 
    // App message handler
    //
    function AppMessageHandler() { };
    AppMessageHandler.prototype = Object.create(kcomm.MessageHandler);
    AppMessageHandler.prototype.canHandle = function (jsonObject) {
        if (jsonObject.Widgets != undefined) {
            appMessageCounter++;
            return true;
        }

        return false;
    };
    AppMessageHandler.prototype.handle = function (jsonObject) {
        if (jsonObject != undefined && !ignoreNextAppMessageToggle) {
            _removeThemeMock();

            // Populate the widgetIds and see if there's a new one
            var widgets = jsonObject.Widgets;
            var pushCount = 0;

            if (widgets) {
                for (var i = 0; i < widgets.length; i++) {
                    if ($.inArray(widgets[i].Id, widgetIds) < 0) {
                        widgetIds.push(widgets[i].Id);
                        pushCount++;
                    }
                }

                // Set the newest widget icon selected in case
                // it came in new
                if (appMessageCounter > 1 && pushCount > 0) {
                    model.get(thelper.TEMPLATE_KEYS.home).currentWidget = widgetIds[widgetIds.length - 1];
                }

                // Show the view
                widman.showWithModel(JSON.stringify(jsonObject),
                    model.get(thelper.TEMPLATE_KEYS.home),
                    thelper.TEMPLATE_KEYS.home,
                    function () {
                        if (widgetTemplate) {
                            widgetTemplate.init(function () {
                                // Apply the selected icon style on the
                                // currently selected widget
                                $.each($(".homeIcon"), function (index, value) {
                                    $value = $(value);

                                    // If the current widget's id matches the currently
                                    // selected widget cached value, add selected class
                                    if (model.get(thelper.TEMPLATE_KEYS.home).currentWidget === $value.attr("data-id")) {
                                        $value.find("img").addClass("homeIconSelected");
                                    }
                                });

                                // All widget icons click handler
                                $document.off("click", ".homeIcon:not([data-type=settings],[data-type=share],[data-type=new_widget])");
                                $document.on("click", ".homeIcon:not([data-type=settings],[data-type=share],[data-type=new_widget])",
                                function (event) {
                                    $this = $(this);
                                    var id = $this.attr("data-id");
                                    var type = $this.attr("data-type");

                                    if (id && type) {
                                        model.get(thelper.TEMPLATE_KEYS.home).currentWidget = id;
                                        _widgetClicked(id, type);
                                        $(".homeIcon").find("img").removeClass("homeIconSelected");
                                        $this.find("img").addClass("homeIconSelected");
                                    }
                                });

                                // Add the new widget button
                                //_addNewWidgetButton();
                            });
                        }
                    });
            }
        }

        // Make sure to disable to toggle to ignore
        ignoreNextAppMessageToggle = false;
    };


    // ----------------------------------------------------------------------
    // 
    // Appearance message handler
    //
    function AppearanceMessageHandler() { };
    AppearanceMessageHandler.prototype = Object.create(kcomm.MessageHandler);
    AppearanceMessageHandler.prototype.canHandle = function (jsonObject) {
        if (jsonObject.Theme != undefined) {
            return true;
        }

        return false;
    };
    AppearanceMessageHandler.prototype.handle = function (jsonObject) {
        if (jsonObject != undefined) {
            // Theme alone was updated
            if (jsonObject.Background == undefined) {
                // Don't add the themeMock again if already added
                if ($("#themeMock").length === 0) {
                    $.ajax({
                        url: "/phone/scripts/designSupport/themeMock.html",
                        type: 'GET'
                    }).success(function (data) {
                        $("#content").append("<div id='themeMock'></div>");
                        $themeMock = $("#themeMock");
                        $themeMock.append(data);
                        $themeMock.hide().fadeIn();
                    });
                }
            }
            // Theme and other appearance parameters were updated
            else {
                // Re-build the grid of the home page
                if (widgetTemplate) {
                    _removeThemeMock();

                    // Rebuild the UI of the home page
                    widgetTemplate.init();
                }
            }

            // Animate fade-out and in
            $overlay = $("#themeMockOverlay");

            if ($overlay.length > 0) {
                $overlay.animate({ opacity: 1 }, 400,
                    function () {
                        _refreshTheme();

                        // Animate back in again
                        $overlay.animate({ opacity: 0 }, 100);
                    });
            } else {
                _refreshTheme();
            }
        }
    };


    // ----------------------------------------------------------------------
    // 
    // Widget Message Handler
    //
    function WidgetMessageHandler() { };
    WidgetMessageHandler.prototype = Object.create(kcomm.MessageHandler);
    WidgetMessageHandler.prototype.canHandle = function (jsonObject) {
       //console.log(JSON.stringify(jsonObject));

        if (jsonObject.Info !== undefined && jsonObject.Info.Type !== undefined) {
            return true;
        }

        return false;
    };
    WidgetMessageHandler.prototype.handle = function (jsonObject) {
        _removeThemeMock();

        var type = (jsonObject.Info.Type + '').toLowerCase();

        // Now show the widget content
        widman.showWithModel(JSON.stringify(jsonObject), model.get(type), type);
    };

    // ----------------------------------------------------------------------
    // 
    // App Branding Message Handler
    //
    function AppBrandingMessageHandler() { };
    AppBrandingMessageHandler.prototype = Object.create(kcomm.MessageHandler);
    AppBrandingMessageHandler.prototype.canHandle = function (jsonObject) {
        if (jsonObject.hasOwnProperty("IconUrl")
        || jsonObject.hasOwnProperty("LoadingImage")
        || jsonObject.hasOwnProperty("LogoUrl")) {
            return true;
        }

        return false;

    };
    AppBrandingMessageHandler.prototype.handle = function (jsonObject) {
        _removeThemeMock();

        if (jsonObject.hasOwnProperty("LoadingImage")) {
            var loadingImgURl = jsonObject.LoadingImage.DefaultLoadingImageUrl;

            if (utility.isValidUrl(loadingImgURl)) {
                window.splashImgUrl = loadingImgURl;
            }
        }

        if (jsonObject.hasOwnProperty("IconUrl")
            && !jsonObject.hasOwnProperty("Carousel")) {

            if (utility.isValidUrl(jsonObject.IconUrl)) {
               
                $("#content:first").hide();

                var $simMenuIconCt = $("#simMenuIconCt:first");
                if ($simMenuIconCt.length == 0) {
                    $simMenuIconCt = $(document.createElement('div'));
                    $simMenuIconCt[0].id = 'simMenuIconCt';
                    $simMenuIconCt[0].className = 'simMenuIcon';
                    $simMenuIconCt.css('background-image', 'url(/phone/images/Luncher_Icon_Screen.png)');
                    var $apphomeIcon = $(document.createElement('div'));
                    $apphomeIcon[0].className = 'apphomeIcon';

                    var applauncherIcon = document.createElement('div');
                    applauncherIcon.id = 'applauncherIcon';
                    $apphomeIcon.append(applauncherIcon);

                    $simMenuIconCt.append($apphomeIcon);

                    $('body:first').append($simMenuIconCt);

                  
                }
                $("#applauncherIcon:first").css('background-image', 'url("' + jsonObject.IconUrl + '")');
                $simMenuIconCt.fadeIn("10000").fadeOut(5000, function () {
                     $("#content").show();
                 });
                
            }
        }

        // If the user updated the logo, request the home
        // so that it is updated on the emulator
        if (jsonObject.hasOwnProperty("LogoUrl")
        && jsonObject.hasOwnProperty("Carousel")) {
            ignoreNextAppMessageToggle = false;
            var message = '{"Instruction":{"Action":"Home"}}';
            kcomm.sendMessage(message, parent.window);
        }
    };

    // On document load, register the widget message handler and start listening
    $(function () {
        // Exclude list so that designModeOverrides won't be added
        var noOverride = [
            thelper.TEMPLATE_KEYS.mediaPlayer,
            thelper.TEMPLATE_KEYS.suspended,
            thelper.TEMPLATE_KEYS.menu
        ];

        // Inject the design mode override scripts and css files
        for (var key in thelper.TEMPLATES) {
            if (thelper.TEMPLATES[key].js && noOverride.indexOf(key) < 0) {
                thelper.TEMPLATES[key].js.push('designModeOverrides.js');
            }
        }

        // Register the message handlers
        kcomm.registerHandler(new AppMessageHandler());
        kcomm.registerHandler(new WidgetMessageHandler());
        kcomm.registerHandler(new AppearanceMessageHandler());
        kcomm.registerHandler(new AppBrandingMessageHandler());

        // Start listening
        kcomm.listen();

        // Pull the last message from the parent window
        // this will most definitly be the app message
        // this is a fix for the case where the iframe loads
        // after the message had been sent
        AppMessageHandler.prototype.handle(window.top.webAppContext.lastPostedMessage);

        // If the home page is not showing, request it
        // this is a possible case when reloading the page in 
        // appearance editing mode on the content editor
        if ($("#homeHero").length <= 0) {
            var message = '{"Instruction":{"Action":"Home"}}';
            kcomm.sendMessage(message, parent.window);
        }
    });


    // Public
    return {
        resetIgnoreNextAppMessageToggle: function () {
            ignoreNextAppMessageToggle = false;
        },
        removeThemeMock: _removeThemeMock
    }
});