﻿var restClientTools = {
 isProd: function () { return typeof(ENV) != "undefined" && ENV  == "prod";}
 , isMobile: function () { return typeof (device) != "undefined" && device.platform && (device.platform.toLowerCase() == 'android' || device.platform.toLowerCase() == 'ios'); }
    , useCDN: function () { return this.isProd() && this.isMobile(); }
};

define('RestClient',
['jquery', 'utility', 'RestStatusCode'],
function ($, utility, restStatusCode) {
    var _defaultSettings = {
            domain: '/api',
            serviceUrl: undefined,
            Id: undefined,
            data: undefined,
            query: undefined,
            success: undefined,
            error: function (jqXHR, textStatus, errorThrown) {
                restStatusCode.execCallback(jqXHR, textStatus, errorThrown);
            },
            ajaxStart: undefined,
            ajaxEnd: undefined
        },
        _getContentType = function () {
            return "json";
        },
        _request = function (urlFormat, type, settings) {
            var url;
            settings = $.extend({}, _defaultSettings, settings);

            if (settings.Id) {
                url = settings.domain + utility.formatMessage(urlFormat, settings.serviceUrl, _getContentType(), settings.Id);
            } else {
                url = settings.domain + utility.formatMessage(urlFormat, settings.serviceUrl, _getContentType());
            }

            if (settings.query) {
                url = url + "?" + $.param(settings.query);
            }

            var actualType = type === 'DELETE' ? 'GET' : type;

            var headers = {};

            if (type === 'DELETE') {
                headers = {
                    'X-HTTP-Method-Override': 'DELETE'
                };
            }

           
            // using settings.data directly will throw an exception if the model has knockout mapping option because it will try to execute it again without passing required parameters
            var data = settings.data ? JSON.stringify(settings.data, null, 2) : undefined;
            
            /// check CDN override
            var useCDN = restClientTools.useCDN();
            if (useCDN && url.indexOf('widget.json') > 0)
                url = 'http://cdn.myapp.kaleoapps.com' + url; /// ignores query strings           
            else
                url = environments[ENV].host + url;

            $.ajax({
                url: url,
                type: actualType,
                dataType: "json",
                data: data,
                cache: false,
                headers: headers,
                beforeSend: function () {
                    if (settings.ajaxStart) {
                        settings.ajaxStart();
                    }
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    if (settings.error) {
                        settings.error(jqXHR, textStatus, errorThrown);
                    }
                },
                success: function (data, textStatus, jqXHR) {
                    if (settings.success) {
                        settings.success(data, textStatus, jqXHR);
                    }
                },
                complete: function (jqXHR) {
                    if (settings.ajaxEnd) {
                        settings.ajaxEnd(jqXHR);
                    }
                }
            });
        },
        list = function (settings) {
            _request("{0}.{1}/", "GET", settings);
        },
        read = function (settings) {
            if (settings.Id) {
                _request("{0}.{1}/{2}", "GET", settings);
            } else {
                _request("{0}", "GET", settings);
            }
        },
        save = function (settings) {
            _request("{0}.{1}/{2}", "PUT", settings);
        },
        create = function (settings) {
            _request("{0}.{1}/", "POST", settings);
        },
        readPage = function (settings) {
            _request("{0}.{1}/query", "GET", settings);
        },
        del = function (settings) {
            _request("{0}.{1}/{2}", "DELETE", settings);
        };

    return {
        list: list,
        get: read,
        save: save,
        create: create,
        readPage: readPage,
        del: del
    };
});