﻿define('RestStatusCode', [], function () {
    /**
     * General error handler.
     */
    var _generalHandler = function (jqXHR, textStatus, errorThrown, message) {
        console.error(errorThrown, jqXHR.status);
        navigator.notification.alert(message,
            function () { },
            jqXHR.statusText,
            "Ok");
    };

    /**
     * Suspended app handler
     */
    var _suspendedAppHandler = function (jqXHR, textStatus, errorThrown, message) {
        console.error(errorThrown, jqXHR.status);

        require(['WidgetManager', 'TemplatesHelper'],
            function (manager, thelper) {
                manager.showWithoutModel(thelper.TEMPLATE_KEYS.suspended);
            });
    };

    /**
     * Status codes expected in the application.
     */
    var _statusCodes = [
        {
            statusCode: 200,
            callback: undefined
        },
        {
            statusCode: 400,
            callback: function (jqXHR, textStatus, errorThrown) {
                if (jqXHR.responseJSON && jqXHR.responseJSON.Message) {
                    console.error(jqXHR.responseJSON.Message, 'Error');
                } else {
                    console.error(errorThrown, jqXHR.status);
                }
            }
        },
        {
            statusCode: 401,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 403,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 404,
            callback: function (jqXHR, textStatus, errorThrown) {
                require(['WidgetManager', 'TemplatesHelper'],
                function (manager, thelper) {
                    $("#content").empty();
                    manager.show(0, thelper.TEMPLATE_KEYS.home);
                });

                console.error(errorThrown, jqXHR.status);
            }
        },
        {
            statusCode: 406,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 407,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 410,
            callback: function (jqXHR, textStatus, errorThrown) {
                _suspendedAppHandler(jqXHR, textStatus, errorThrown, 'App has been suspended');
            }
        },
        {
            statusCode: 500,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 501,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: 504,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        },
        {
            statusCode: undefined,
            callback: function (jqXHR, textStatus, errorThrown) {
                _generalHandler(jqXHR, textStatus, errorThrown, 'An error has occurred!');
            }
        }];

    /**
     * The callback to handler all status codes previously defined.
     */
    var _execCallback = function (jqXHR, textStatus, errorThrown) {
        var filteredCodes = $.grep(_statusCodes, function (item) {
            return item["statusCode"] == jqXHR.status;
        });

        if (filteredCodes && filteredCodes.length > 0 && $.isFunction(filteredCodes[0].callback)) {
            filteredCodes[0].callback(jqXHR, textStatus, errorThrown);
        }
    };

    return {
        statusCodes: _statusCodes,
        execCallback: _execCallback
    };
});