﻿/**
 * This module provides a set of utility properties and methods for use with template related operations.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 * Created: 28 Aug, 2013
 * Updated: 4/2/2013 Danny CDN
 * Updated: 20 May, 2014 Waleed removed CDN code now using bundle
 */
define('TemplatesHelper', ['jquery'], function ($) {
    var _THEME_COLOR_KEYS = {
        Background: "Background",
        MainSectin: "MainSection",
        MainText: "MainText",
        WidgetBackground: "WidgetBackground",
        SubSectionText: "SubSectionText",
        SubSection: "SubSection",
        SubText: "SubText",
        Text: "Text",
        Title: "Title"
    };

    var _THEME_BACKGROUND_KEYS = {
        AppBackgroundColor: "Color",
        AppBackgroundImageDefault: "DefaultImageUrl",
        AppBackgroundImageTablet: "TabletImageUrl",
        AppBackgroundImageIPhone: "IPhoneImageUrl",
        AppBackgroundImageIPad: "IPadImageUrl",
        AppBackgroundImageRepeat: "BackgroundRepeat",
        AppBackgroundGradient: "Gradient",
        AppBackgroundSize: "Size"
    };

    var _THEME_COLOR_DEFAULTS = [];
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.Background] = "#319dc4";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.MainSectin] = "#fafcfb";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.MainText] = "rgb(36, 109, 130)";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.WidgetBackground] = "#121212";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.SubSectionText] = "#9ea5a9";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.SubSection] = "#edf3f3";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.SubText] = "rgb(149, 157, 160)";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.Text] = "#858585";
    _THEME_COLOR_DEFAULTS[_THEME_COLOR_KEYS.Title] = "white";

    var _THEME_BACKGROUND_DEFAULTS = [];
    _THEME_BACKGROUND_DEFAULTS[_THEME_BACKGROUND_KEYS.AppBackgroundColor] = "#121212";
    _THEME_BACKGROUND_DEFAULTS[_THEME_BACKGROUND_KEYS.AppBackgroundImage] = "";
    _THEME_BACKGROUND_DEFAULTS[_THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat] = "no-repeat";
    _THEME_BACKGROUND_DEFAULTS[_THEME_BACKGROUND_KEYS.AppBackgroundGradient] = "";

    var _ICON_STYLES = [
        "icon-twitter", "icon-facebook", "icon-home", "icon-linkedin", "icon-facebook",
        "icon-instagram", "icon-google-plus", "icon-mail", "icon-phone", "icon-photo",
        "icon-earth", "icon-location", "icon-settings", "icon-menu", "icon-instagram",
        "icon-share", "icon-settings", "icon-video", "icon-music", "calendar-empty"
    ];

    /**
     * The keys for fetching templates are defined as an object.
     *
     * The paths of the widget templates are defined as an array
     * for easier lookup.
     */
    var _TEMPLATE_KEYS = {
        core: 'core',
        menu: 'menu',
        mediaPlayer: 'mediaPlayer',
        suspended: 'suspended',
        home: 'home',
        text: 'text',
        contactus: 'contactus',
        webview: 'webview',
        facebook: 'facebook',
        twitter: 'twitter',
        instagram: 'instagram',
        places: 'places',
        groups: 'groups',
        mediacenter: 'rss',
        listView: 'listview',
        event: 'event',
        aboutUs: 'aboutus',
        socialWall: 'socialwall',
        settings: 'settings',
        share: "share"
    };

    var _TEMPLATES = [];
    _TEMPLATES[_TEMPLATE_KEYS.core] = {
        css: ['phone.css', 'defaultTheme.css', 'owl.carousel.css',
              'owl.theme.css', 'owl.transitions.css', 'iconmoon.css', 'phone2.css']
    };

    _TEMPLATES[_TEMPLATE_KEYS.menu] = {
        page: 'tmp_menu.html'
    };

    _TEMPLATES[_TEMPLATE_KEYS.mediaPlayer] = {
        page: 'tmp_media_player.html',
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.suspended] = {
        page: 'tmp_suspended.html'
    };

    _TEMPLATES[_TEMPLATE_KEYS.home] = {
        page: 'tmp_home.html',
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.text] = {
        page: 'tmp_text.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js', 'frag_social.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.contactus] = {
        page: 'tmp_contact_us.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js', 'frag_social.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.webview] = {
        page: 'tmp_web_view.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };
    _TEMPLATES[_TEMPLATE_KEYS.facebook] = {
        page: 'tmp_web_view.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };
    _TEMPLATES[_TEMPLATE_KEYS.twitter] = {
        page: 'tmp_web_view.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };
    _TEMPLATES[_TEMPLATE_KEYS.instagram] = {
        page: 'tmp_web_view.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.places] = {
        page: 'tmp_places.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.groups] = {
        page: 'tmp_groups.html',
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.mediacenter] = {
        page: 'tmp_media_center.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.event] = {
        page: 'tmp_events.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.listView] = {
        page: 'tmp_list_view.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js', 'frag_social.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.aboutUs] = {
        page: 'tmp_about_us.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.socialWall] = {
        page: 'tmp_social_wall.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.settings] = {
        page: 'tmp_settings.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    _TEMPLATES[_TEMPLATE_KEYS.share] = {
        page: 'tmp_share.html',
        fragments: ['frag_header.html'],
        js: ['tmp_widgets.js']
    };

    /**
     * Is the current system a mobile phone.
     */
    var _isMobile = function () {
        var platform = device.platform;

        if (platform) {
            platform = platform.toLowerCase();
            return ((platform == 'android') || (platform == 'ios'));
        } else {
            return false;
        }
    };

    /**
     * Returns true if the checked template control exists.
     */
    var _templateControlExists = function (controlId) {
        return $(controlId)[0] !== undefined;
    };

    /**
     * Adds a new script element to the current document.
     *
     * @param src The source path of the script to add
     * @param target
     * @param onloadHandler
     */
    var _addScript = function (src, target, onloadHandler) {
        var eScript = document.createElement('script');
        eScript.setAttribute('type', 'text/javascript');

        if (src != undefined) {
            eScript.setAttribute('src', src);
        }

        // This is an id for easy clean up of script tags when loading new widgets
        if (target) {
            document.getElementById(target).appendChild(eScript);
        } else {
            var head = document.head || document.getElementsByTagName('head')[0];
            head.appendChild(eScript);
        }

        if (onloadHandler) {
            eScript.onreadystatechange = function () {
                if (this.readyState == 'complete') {
                    onloadHandler();
                }
            };

            eScript.onload = function () {
                onloadHandler();
            }
        }
    };

    /**
     * Adds a new script element to the current document.
     *
     * @param src The source path of the script to add
     * @param onloadHandler
     */
    var _loadScript = function (src, onloadHandler) {
        $.ajax({
            url: src,
            async: false
        }).success(function () {
            if (onloadHandler) {
                onloadHandler();
            }
        });
    };

    /**
     * Adds a new style link element to the current document.
     * 
     * @param target The target Id of the element under which to add the new style
     * @param src The source path of the script to add
     */
    var _addStyleLink = function (target, src) {
        var eLink = document.createElement('link');
        eLink.setAttribute('rel', 'stylesheet');
        eLink.setAttribute('type', 'text/css');

        if (src != undefined) {
            eLink.setAttribute('href', src);
        }

        // This is an id for easy clean up of script tags when loading new widgets
        document.getElementById(target).appendChild(eLink);
    };

    /**
     * Adds a new style element to the current document.
     * 
     * @param id The Id of the new tag
     * @param targetId (OPTIONAL) The Id of the element under which to create the new tag
     */
    var _addStyleTag = function (id, targetId) {
        var eLink = document.createElement('style');
        eLink.setAttribute('id', id);

        if (targetId) {
            document.getElementById(targetId).appendChild(eLink);
        } else {
            var head = document.head || document.getElementsByTagName('head')[0];
            head.appendChild(eLink);
        }
    };

    /**
     * Executes asynchronous.
     *
     * @param iterations The number of iterations to execute
     * @param func The function to execute at each iteration
     * @param callback The callback to invoke once the iterations are complete
     */
    function asyncLoop(iterations, func, callback) {
        var index = 0;
        var done = false;
        var loop = {
            /**
             * Advance the loop one position.
             */
            next: function () {
                if (done) {
                    return;
                }

                if (index < iterations) {
                    index++;
                    func(loop);
                } else {
                    done = true;
                    callback();
                }
            },

            /**
             * Get the current iteration index.
             */
            iteration: function () {
                return index - 1;
            },

            /**
             * Breaks the loop.
             */
            break: function () {
                done = true;
                callback();
            }
        };

        loop.next();
        return loop;
    }

    /**
     * Returns a resource from the defined resources based on id and type.
     *
     * @param type Resource type
     * @param id The id of the resource to fetch
     */
    var _getResourceURI = function (type, id) {
        var deferred = $.Deferred();
        var selection = undefined;
        var resource = undefined;
        var livePath = undefined;

        // Select the resource according to the criteria
        // Template resources are all placed within the templates path
        if (type == 'page' || type == 'fragments' || type == 'js' || type == 'css') {
            selection = _TEMPLATES[id][type];

            // XXX The if condition is here for LEGACY bundles
            if (window.templatesFallbackUrl) {
                livePath = window.templatesFallbackUrl;
            } else {
                livePath = window.templatesUrl;
            }
        }
        // Core CSS resource used for the core modules
        else if (type == 'coreCss') {
            selection = _TEMPLATES[id].css;
            livePath = window.cssUrl;
        } else {
            console.error('Un-identified resource type: ' + type);
        }

        if (selection != undefined) {
            // Dealing with an array of resources
            if (selection instanceof Array) {
                resource = [];

                asyncLoop(selection.length, function (loop) {
                    var i = loop.iteration();

                    // Avoid caching resources that are specifically referenced from 
                    // a remote server or a CDN
                    if (selection[i].substring(0, 7) == 'http://'
                        || selection[i].substring(0, 8) == 'https://') {
                        resource.push(selection[i]);
                        loop.next();
                    } else {
                        resource.push(livePath + selection[i]);
                        loop.next();
                    }
                }, function () {
                    deferred.resolve(resource);
                });
            }
                // Single resource
            else {
                // Avoid caching resources that are specifically referenced from 
                // a remote server or a CDN
                if (selection.substring(0, 7) == 'http://'
                    || selection.substring(0, 8) == 'https://') {
                    deferred.resolve(selection);
                } else {
                    deferred.resolve(livePath + selection);
                }
            }
        }
            // Mark the deferred as done anyway, since we did not load any resources
        else {
            deferred.resolve();
        }

        return deferred;
    };

    /**
     * Fetches a resource in text form.
     * 
     * @param uri The URI of the resource
     * @param callback A callback to execute when done
     */
    var _getResourceInText = function (uri, callback) {
        $.ajax({
            async: false,
            url: uri,
            type: 'GET'
        }).done(function (data) {
            if (callback) {
                callback(data)
            }
        });
    };

    /**
     * Applys the theme data to the main application theme CSS file.
     * 
     * @param css The css file to apply the theme onto place holders should exist '@Theme.<Key>'
     * @param appearanceModel The application appearance model, theme data should be under
     *                        appearanceModel.Theme[].<Key>
     */
    var _applyTheme = function (css, appearanceModel) {
      
        var result = css;

        // Process the color theme
        for (var key in _THEME_COLOR_DEFAULTS) {
            if (_THEME_COLOR_DEFAULTS.hasOwnProperty(key)) {
                var entry = _getThemeValue(key, appearanceModel.Theme, _THEME_COLOR_DEFAULTS);
                result = result.replace(new RegExp("@Theme_" + key + "@", "g"), entry);
            }
        }

        // Icon Text Color
        var iconTextColor = _getThemeValue("IconFontColor", appearanceModel);

        if (!iconTextColor || iconTextColor.trim().length <= 0) {
            fontType = "black";
        }

        result = result.replace(new RegExp("@Theme_IconFontColor@", "g"), iconTextColor);

        // Icon Color
        var iconColor = _getThemeValue("IconColor", appearanceModel);

        if (!iconColor || iconColor.trim().length <= 0) {
            iconColor = "black";
        }

        result = result.replace(new RegExp("@Theme_IconColor@", "g"), iconColor);

        // Font
        var fontType = _getThemeValue("FontType", appearanceModel);

        if (!fontType || fontType.trim().length <= 0) {
            fontType = "'Helvetica Neue', Helvetica, arial, verdana";
        }

        result = result.replace(new RegExp("@Theme_Font_FontType@", "g"), fontType);

        // BACKGROUND
        //
        // Background image default
        var bgImgDefault = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundImageDefault, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageDefault + "@", "g"), bgImgDefault);

        // Background image iphone
        var bgImgIphone = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundImageIPhone, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageIPhone + "@", "g"), bgImgIphone);

        // Background image ipad
        var bgImgIpad = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundImageIPad, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageIPad + "@", "g"), bgImgIpad);

        // Background image android tablet
        var bgImgTab = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundImageTablet, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageTablet + "@", "g"), bgImgTab);

        // Background color
        var bgColor = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundColor, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundColor + "@", "g"), bgColor);

        // Background gradient
        var bgGradient = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundGradient, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundGradient + "@", "g"), bgGradient);

        // Background repeat
        var bgRepeat = _getThemeValue(_THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat, appearanceModel.Background, _THEME_BACKGROUND_DEFAULTS);
        bgRepeat = bgRepeat.trim().toLowerCase();

        if (bgRepeat === 'horizontally') {
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat + "@", "g"), "repeat-x");
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundSize + "@", "g"), "");
        } else if (bgRepeat === 'vertically') {
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat + "@", "g"), "repeat-y");
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundSize + "@", "g"), "");
        } else if (bgRepeat === 'horizontallyandvertically') {
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat + "@", "g"), "repeat");
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundSize + "@", "g"), "");
        } else if (bgRepeat === 'none') {
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundImageRepeat + "@", "g"), "no-repeat");
            result = result.replace(new RegExp("@Theme_BG_" + _THEME_BACKGROUND_KEYS.AppBackgroundSize + "@", "g"), "cover");
        }

        //Restart the carousel
        carouselPhone.restart({ containerSelector: "#homeHero" });

        return result;
    };

    /**
     * Applys icon related styles.
     */
    var _applyIconStyles = function () {
        require(['LocalModel'], function (model) {
            for (var i = 0; i < _ICON_STYLES.length; i++) {
                var $iconClass = $("[class*=" + _ICON_STYLES[i] + "]");

                if ($iconClass.length > 0) {
                    var iconStyle = "";

                    if (model.appearanceModel() && model.appearanceModel().IconStyle) {
                        iconStyle = model.appearanceModel().IconStyle;
                    }

                    $iconClass.removeClass(_ICON_STYLES[i]).addClass(_ICON_STYLES[i] + iconStyle);

                    if (model.appearanceModel()) {
                        $iconClass.css("color", model.appearanceModel().IconColor);
                    }
                }
            }
        });
    };

    /**
     * Getter for theme values, you should provide a key and the model
     * to fetch the value from.
     *
     * There's an optional defaultArray param for fetching the default
     * value for the key in case it was undefined in the model.
     *
     * @param key The key to look for
     * @param model The model to fetch the value from
     * @param defaultsArray The defaults array
     */
    var _getThemeValue = function (key, model, defaultsArray) {
        var result = "";

        // If the model is present, fetch the value from it
        if (model) {
            result = model[key];

            if (defaultsArray) {
                // No value, fetch the default
                if (!result || result.trim().length <= 0) {
                    result = defaultsArray[key];
                    //} else {
                    // Always update the defaults array with the value
                    // fetched from the model, in case we need to re-populate the
                    // theme in cases where the model is missing
                    //defaultsArray[key] = result;
                }
            }
        } else {
            // Read from the defaults array
            result = defaultsArray[key];
        }

        return result;
    };

    // Expose
    return {
        templateControlExists: _templateControlExists,
        addScript: _addScript,
        loadScript: _loadScript,
        addStyleLink: _addStyleLink,
        addStyleTag: _addStyleTag,
        getResourceURI: _getResourceURI,
        getResourceInText: _getResourceInText,
        isMobile: _isMobile,
        TEMPLATE_KEYS: _TEMPLATE_KEYS,
        TEMPLATES: _TEMPLATES,
        applyTheme: _applyTheme,
        applyIconStyles: _applyIconStyles
    }
});