window.onerror = function myErrorHandler(errorMsg, url, lineNumber) {
    alert("Error occured: " + errorMsg);//or any message
    return false;
}

/**
 * This module is responsible for bootstrapping the core application resources.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 */
var bootstrapper = (function () {
    "use strict";

    // An array of all script files to download
    var SCRIPTS = ['RestClient.js',
        'RestStatusCode.js',
        'DataCacheManager.js',
        'jquery.js',
        'DataFetcher.js',
        'fastclick.js',
        'kcomm.js',
        'knockout.js',
        'knockout.mapping.js',
        'LocalModel.js',
        'utility.js',
        'TemplateLoader.js',
        'TemplatesHelper.js',
        'WidgetManager.js',
        'maps.js',
        'kPush.js',
        'kAPI.js',
        'xml2json.min.js',
        'owl.carousel.min.js',
        'carouselHelper.js',
        'jquery.cookie.js'
    ];

    // File download queue
    var DOWNLOAD_QUEUE = [];

    /**
     * Request the app version from the server, and if there is a difference
     * call the passed callback function
     *
     * @param callback The callback function to invoke when there a diff
     */
    function _versionCheck(onDiff, onComplete) {

        var url = window.host + '/api/designer/appVersion.json/' + window.appId;
        var xmlhttp = new XMLHttpRequest();
        xmlhttp.open("GET", url, false);
        xmlhttp.setRequestHeader("Content-Type", "application/json");

        xmlhttp.onreadystatechange = function () {
            if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
                var buildVersion = localStorage['buildVersion'];
                var jsonObject = JSON.parse(xmlhttp.responseText);

                console.log('<BOOTSTRAPPER> Cached build version: ' + buildVersion);
                console.log('<BOOTSTRAPPER> Server build version: ' + jsonObject.ServerBuildVersion);
                console.log('<BOOTSTRAPPER> Server data version: ' + jsonObject.LiveContentVersion);

                if (jsonObject != undefined && buildVersion != jsonObject.ServerBuildVersion) {
                    // Reset the cache
                    var localAppId = localStorage.getItem("appID"); // HACK: keep track of appID, appID will not change.
                    localStorage.clear();
                    if (localAppId) {
                        localStorage.setItem("appID");
                    }
                    localStorage.setItem('buildVersion', jsonObject.ServerBuildVersion);
                    localStorage.setItem('contentVersion', jsonObject.LiveContentVersion);

                    // Download all scripts all over again
                    onDiff();
                }

                onComplete();
            }
        }

        xmlhttp.send();
    }

    /**
     * Downloads the application's JS resources.
     * This method will first check for version differences
     * then if there's a diff, it will delete the current app
     * folder and then finally download all over again.
     */
    function _downloadAndProceed() {
        // Delete the app dir
        fsManager.deleteAppDir(
            function () {
                // Add all scripts to the download queue
                for (var i = 0; i < SCRIPTS.length; i++) {
                    if (SCRIPTS[i] && SCRIPTS[i].replace(/\s+/g, '').length > 0) {
                        DOWNLOAD_QUEUE.push(SCRIPTS[i]);
                    }
                }

                // Start the download
                _downloadRecursive();
            }, undefined);
    }

    /**
     * The recursive download method
     */
    function _downloadRecursive() {

        // No more files in the queue, proceed to the home page
        if (DOWNLOAD_QUEUE.length == 0) {
            _proceed();
        }

        // Pop the current file from the queue
        var file = DOWNLOAD_QUEUE.pop();

        // Download it
        if (file && file.replace(/\s+/g, '').length > 0) {
            fsManager.downloadResource(window.jsDownloadUrl + file,
               fsManager.DIR_SCRIPTS, file, function () {
                   _downloadRecursive();
               }, undefined);
        }
    }

    /**
     * Proceed to the home page.
     */
    function _proceed() {

        // Acquire the app root path, then redirect to the next page
        fsManager.getAppCacheRootDir(function (path) {
            localStorage.fsPath = path.fullPath.replace('file://', '') + '/';
            window.location = 'main.html';
        });
    }

    /**
     * Initialize the application.
     */
    function _init() {
        //_downloadAndProceed();
        _proceed();
        /*
        _versionCheck(
            // On version difference
            function () {
                _downloadAndProceed();
            },
            // On version not changed
            function () {
                // Make sure to check for missing files before proceeding
                fsManager.checkForMissingFiles(fsManager.DIR_SCRIPTS, SCRIPTS, function () {
                    _proceed();
                }, function () {
                    _downloadAndProceed();
                });
            }
        );
		*/
    }

    return {
        initialize: _init
    }
})();