﻿/**
 * This library facilitates communication with our NodeJS API server.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 */
var kAPI = (function () {
    // API call timeout (3 seconds)
    var _apiTimeout = 3000;

    // The API server address
    var _server = window.apiServer;
    console.log('kpush server api: ' + _server);

    /**
     * Call the API server.
     *
     * @param id The id to pass to the API server
     * @param method The method to call on the API server
     * @param params The params to pass to the API server method
     * @param callback The callback to execute after the API call ends
     * @param errorCallback A callback to call in case of failure
     */
    var _call = function (id, method, params, callback, errorCallback) {
        try {
            if (typeof (params) == "string") {
                params = JSON.parse(params);
            }

            if (params != null) {
                if (params.appID)
                    params.appID = params.appID.toString();
                else
                    params.appID = appId.toString();
            }

            var data = {
                id: id,
                method: method,
                params: params,
                apiKey: window.appId
            };

            var tmr = null;

            var p = $.getJSON(_server + "?callback=?", { data: JSON.stringify(data) }, function (result) {
                if (tmr) {
                    clearTimeout(tmr);
                }

                if (result.error) {
                    if (console.error) {
                        console.error("Error: " + JSON.stringify(result));
                    }

                    if (result.error.code == 1500) {
                        _accessDeniedHandle(result);
                    }
                }

                if (callback) {
                    callback(result);
                }
            },
            function (e) {
                if (errorCallback) {
                    errorCallback(e);
                } else if (console.error) {
                    console.error(JSON.stringify(e));
                    callback({ error: { code: -1, message: "Error communicating with the server" } });
                }
            });

            tmr = setTimeout(function () {
                if (p) {
                    p.abort();

                    if (errorCallback) {
                        errorCallback({ error: { code: 0, message: "timeout" } });
                    }
                }
            }, _apiTimeout);
        } catch (e) {
            if (errorCallback) {
                errorCallback(e);
            } else if (console.log) {
                console.error(e.message);
            }
        }
    };

    /**
     * A handler function for cases where access is denied.
     *
     * @param err The error object
     */
    var _accessDeniedHandle = function (err) {
    };

    // Expose
    return {
        call: _call
    }
})();

if (typeof define === "function" && define.amd) {
    define("kAPI", [], function (cfg) { return kAPI; });
}