﻿/**
 * Provides Push notifications support for the app.
 *
 * Author: Daniel Hindi
 * Updated by: Waleed Madanat
 * Created on: 11/1//13 in the very early AM
 * Requires: phonegap push plugin
 */

var kPush = {
    pushNotification: null,

    deviceId: null,

    // Should be overridden for each specific platform
    notificationHandler: null,
    init: function () {

        try {
            if (typeof (window.plugins) == "undefined" || typeof (window.plugins.pushNotification) == "undefined")
                console.log("Warning: window.plugins.pushNotification == undefined");
            kPush.pushNotification = window.plugins.pushNotification;

            if (device.platform.toLowerCase() == 'android') {
                kPush.notificationHandler = new kPushAndroidNotificationHandler(kPush.pushNotification);
            } else {
                kPush.notificationHandler = new kPushiOSNotificationHandler(kPush.pushNotification);
            }

            kPush.notificationHandler.onRegistered = kPush.registerToken;
            kPush.notificationHandler.register();
        } catch (err) {
            console.error("There was an error on this page.\n\nError description: " + err.message + "\n\n");
        }
    },
    registerToken: function (deviceID) {
        console.log('registerToken: ' + deviceID);

        kPush.deviceId = deviceID;

        if (typeof (appId) == "undefined") {
            alert('kPush: AppID has not been set yet. Subscription has failed.');
        } else {
            kAPI.call(1, 'groups/subscriptions/add',
                {
                    groupID: '',
                    appID: appId + '',
                    deviceID: kPush.deviceId,
                    os: device.platform ? device.platform.toLowerCase() : ''
                },
                function (response) {
                },
                function (err) {
                    console.error(JSON.stringify(err));
                });
        }
    }
};

if (typeof define === "function" && define.amd) {
    define("kPush", [], function () { return kPush; });
}

// - Android --------------------------------------------------------------------
//
function kPushAndroidNotificationHandler(pushNotification) {
    this.pushNotification = pushNotification;
    this.androidSenderID = "515869937012"; // projectID
}

kPushAndroidNotificationHandler.prototype = {
    /**
     * Register the device on the Google Cloud Messaging service (GCM).
     */
    register: function () {
        this.pushNotification.register(this.successHandler, this.errorHandler, { "senderID": this.androidSenderID, "ecb": "kPush.notificationHandler.onNotification" });  // required!
    },
    /**
     * Device registered
     */
    onRegistered: function (deviceID) {
        kPush.deviceId = deviceID;
    },
    /**
     * Handle GCM, this method handles responses to registration and notifications
     */
    onNotification: function (e) {
        switch (e.event) {
            case 'registered':
                if (e.regid.length > 0) {
                    // Your GCM push server needs to know the regid before it can push to this device
                    // here is where you might want to send it the regid for later use.
                    this.onRegistered(e.regid);
                }
                break;

            case 'message':
                // If this flag is set, this notification happened while we were in the foreground.
                // you might want to play a sound to get the user's attention, throw up a dialog, etc.
                if (e.foreground) {
                    //console.log('Forground Message');
                }
                    // otherwise we were launched because the user touched a notification in the notification tray.
                else {
                    if (e.coldstart) {
                        //console.log('Cold start notification');
                    } else {
                        //console.log('Background notification');
                    }
                }

                // Show the message while in background
                navigator.notification.alert(e.payload.message, null, e.payload.title, "OK");
                break;

            case 'error':
                console.error("An error occurred: " + e.msg);
                break;

            default:
                console.error("EVENT -> Unknown, an event was received and we do not know what it is");
                break;
        }
    },
    /**
     * Invoked on success.
     */
    successHandler: function (result) {
        //console.log('success:' + result);
    },
    /**
     * Invoked on failure.
     */
    errorHandler: function (error) {
        console.error('error:' + error);
    }
};

// - iOS ------------------------------------------------------------------------
//
function kPushiOSNotificationHandler(pushNotification) {
    this.pushNotification = pushNotification;
}

kPushiOSNotificationHandler.prototype = {
    /**
     * Device registered
     */
    onRegistered: function (deviceID) {
        kPush.deviceId = deviceID;
    },
    /**
     * Register the device on the Google Cloud Messaging service (GCM).
     */
    register: function () {
        try {
            this.pushNotification.register(this.onRegistered, this.errorHandler, {
                "badge": "true", "sound": "true", "alert": "true",
                "ecb": "kPush.notificationHandler.onNotification"
            }); // required!            
        } catch (err) {
            console.error("There was an error on this page.\n\nError description: " + err.message + "\n\n");
        }
    },
    /**
     * Handle APNS, this method handles responses to registration and notifications
     */
    onNotification: function (e) {
        if (e.alert) {
            navigator.notification.alert(e.alert, null, "Alert", "OK");
        }

        if (e.sound) {
            //var snd = new Media(e.sound);
            //snd.play();
        }

        if (e.badge) {
            this.pushNotification.setApplicationIconBadgeNumber(this.successHandler, e.badge);
        }
    },
    tokenHandler: function (result) {
        // Your iOS push server needs to know the token before it can push to this device
        // here is where you might want to send it the token for later use.
        //this.onRegistered(result);
        kPush.deviceId = result;
    },
    /**
     * Invoked on success.
     */
    successHandler: function (result) {
        this.onRegistered(result);
        //console.log('success:' + result);
    },
    /**
     * Invoked on failure.
     */
    errorHandler: function (error) {
        console.error('ios error:' + error);
    }
};

// Initialize on device ready
document.addEventListener('deviceready', kPush.init, false);