﻿/**
 * Acts as the temporary local model, until Ady enhances the model module.
 *
 * Author: Waleed Madanat
 * Version: 1.0
 * Created: 28 Aug, 2013
 * Updated: 4 Sep, 2013
 */
define('LocalModel',
['knockout', 'TemplatesHelper', 'utility'],
function (ko, thelper, util) {
    // A 2D Array of models per type
    var _Models = [];

    var _APP_MODEL;
    var _APPEARANCE_MODEL;

    // Home
    _Models[thelper.TEMPLATE_KEYS.home] = {
        Id: ko.observable(),
        Name: ko.observable(),
        Widgets: ko.observableArray(),
        AppearanceId: ko.observable(),
        Branding: {
            Id: ko.observable(),
            Carousel: {
                Id: ko.observable(),
                Images: ko.observableArray()
            },
            LogoUrl: ko.observable(),
            IconUrl: ko.observable(),
            DefaultLoadingImageUrl: ko.observable(),
            TabletLoadingImageUrl: ko.observable(),
            IPhoneLoadingImageUrl: ko.observable(),
            IPadLoadingImageUrl: ko.observable()
        }
    };

    // Text
    _Models[thelper.TEMPLATE_KEYS.text] = {
        Id: ko.observable(),
        Content: {
            Text: ko.observable(),
            HeaderTitle: ko.observable()
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    // Web View
    var webViewModel = {
        Id: ko.observable(),
        Content: {
            Url: ko.observable(),
            Popup: ko.observable()
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };
    _Models[thelper.TEMPLATE_KEYS.webview] = new Object(webViewModel);
    _Models[thelper.TEMPLATE_KEYS.facebook] = new Object(webViewModel);
    _Models[thelper.TEMPLATE_KEYS.twitter] = new Object(webViewModel);
    _Models[thelper.TEMPLATE_KEYS.instagram] = new Object(webViewModel);


    // Custom binding to properly load the image carousel
    ko.bindingHandlers.carousel = {
        init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
            // This will be called when the binding is first applied to an element
            // Set up any initial state, event handlers, etc. here
            var value = valueAccessor();

            // Next, whether or not the supplied model property is observable, get its current value
            //            var valueUnwrapped = ko.unwrap(value);

            // Load the carousel once all image divs' have been loaded
            var carouselChecker = setInterval(function () {
                if ($("#" + element.getAttribute('id')).find("img.lazyOwl").length >= allBindings().foreach.data().length) {
                    carouselPhone.restart({ containerSelector: "#" + element.getAttribute('id') });
                    clearInterval(carouselChecker);
                }
            }, 1000);
        },
        update: function (element, valueAccessor, allBindings) {
            // First get the latest data that we're bound to
            var value = valueAccessor();

            // Next, whether or not the supplied model property is observable, get its current value
//            var valueUnwrapped = ko.unwrap(value);

            // Load the carousel once all image divs' have been loaded
            var carouselChecker = setInterval(function () {
                if ($("#" + element.getAttribute('id')).find("img.lazyOwl").length >= allBindings().foreach.data().length) {
                    carouselPhone.restart({ containerSelector: "#" + element.getAttribute('id') });
                    clearInterval(carouselChecker);
                }
            }, 1000);
        }
    };


    // Places
    ko.extenders.showHomePage = function (target, option) {
        target.subscribe(function (newValue) {
            try {
                if (widgetTemplate && widgetTemplate.resetView)
                    widgetTemplate.resetView();

            } catch (err) {
                console.warn('resetView is not defined, but you may safely ignore this warning!' + err);
            }
        });

        return target;
    };

    _Models[thelper.TEMPLATE_KEYS.places] = {
        Id: ko.observable(),
        Content: {
            Sections: ko.observableArray().extend({ showHomePage: null })
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    // Contact US
    ko.extenders.updateMap = function (target, option) {
        target.subscribe(function (newValue) {
            try {
                if (recenterMap) {
                    recenterMap(newValue);
                }
            } catch (err) {
                console.warn('recenterMap is not defined, but you may safely ignore this warning!');
            }
        });

        return target;
    };

    _Models[thelper.TEMPLATE_KEYS.contactus] = {
        Id: ko.observable(),
        Content: {
            Text: ko.observable(),
            Email: ko.observable(),
            ContactNumber: ko.observable(),
            Address: ko.observable().extend({ updateMap: null }),
            ShowAddressInMap: ko.observable(),
            FacebookUrl: ko.observable(),
            TwitterUrl: ko.observable(),
            GooglePlusUrl: ko.observable(),
            InstagramUrl: ko.observable(),
            LinkedInUrl: ko.observable()
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    // Media Center
    _Models[thelper.TEMPLATE_KEYS.mediacenter] = {
        Id: ko.observable(),
        Content: {
            Sections: ko.observableArray().extend({ showHomePage: null })
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    //Events
    _Models[thelper.TEMPLATE_KEYS.event] = {
        Id: ko.observable(),
        Content: {
            Sections: ko.observableArray().extend({ showHomePage: null })
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        },
        // Aliased from the utility module
        getTimeString: function (date) {
            return util.getTimeString(date);
        },
        // Aliased from the utility module
        getDayNumber: function (date) {
            return util.getDayNumber(date);
        },
        // Aliased from the utility module
        getMonthChar: function (date) {
            return util.getMonthChar(date);
        },
        activeEvent: function (date, element) {
          
            var $element = $(element);
            if (date()) {
                var eventDate = new Date(date());
                var currentDate = new Date();

                if (eventDate > currentDate) {
                    return true;
                }
                else {
                    // dont hide it from here since the _resetView
                    // will show it. so it will be hidden from _resetView.
                    $element.attr("expired-event", "expired-event");
                    return false;
                }
            }
            else {
                $element.attr("expired-event", "expired-event");
                return false;
            }
           
        }
    };

    //About Us
    _Models[thelper.TEMPLATE_KEYS.aboutUs] = {
        Id: ko.observable(),
        Content: {
            Sections: ko.observableArray().extend({ showHomePage: null })
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    // List View
    _Models[thelper.TEMPLATE_KEYS.listView] = {
        Id: ko.observable(),
        Content: {
            Carousel: {
                Id: ko.observable(),
                Images: ko.observableArray()
            },
            Widgets: ko.observableArray()
        },
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    // social wall
    _Models[thelper.TEMPLATE_KEYS.socialWall] = {
        Id: ko.observable(),
        Info: {
            Name: ko.observable(),
            Type: ko.observable(),
            Icon: ko.observable(),
            SocialEnabled: ko.observable()
        }
    };

    /**
     * @param widgetType The widget type to get the model for
     */
    var _get = function (widgetType) {
        return _Models[widgetType];
    };

    /**
     * Fetches or updates then returns the appModel.
     *
     * @param newModel (Optional) The new app model
     */
    var _appModel = function (newModel) {
        if (newModel) {
            _APP_MODEL = newModel;
        }

        return _APP_MODEL;
    };

    /**
     * Fetches or updates then returns the appearance model.
     *
     * @param newModel (Optional) The new appearance model
     */
    var _appearanceModel = function (newModel) {
        if (newModel) {
            _APPEARANCE_MODEL = newModel;
        }

        return _APPEARANCE_MODEL;
    };

    return {
        get: _get,
        appModel: _appModel,
        appearanceModel: _appearanceModel
    }
});