﻿var utility = (function () {
    var formatMessage = function () {
        var s = arguments[0];

        for (var i = 0; i < arguments.length - 1; i++) {
            var reg = new RegExp("\\{" + i + "\\}", "gm");
            s = s.replace(reg, arguments[i + 1]);
        }

        return s;
    };

    /**
     * Fetches a query string parameter by name.
     *
     * @param name The query string parameter name
     */
    var _getParameterByName = function (name) {
        name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
        var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
            results = regex.exec(location.search);
        return results == null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
    };

    /**
     * Check if the passed object is a window.
     *
     * Credit to jQuery source: https://github.com/jquery/jquery/blob/master/src/core.js
     */
    var _isWindow = function (obj) {
        var win = false;

        if (obj != null && obj == obj.window) {
            win = true;
        }

        return win;
    };

    /**
     * Check if the provided String represents a JSON object.
     */
    var _isJSON = function (str) {
        var result = false;

        try {
            JSON.parse(str);
            result = true;
        } catch (e) {}

        return result;
    };

    /**
     * Returns a string representation of the provided date object.
     *
     * @param date The date to stringify.
     */
    var _getTimeString = function (date) {
        //Refer to https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toLocaleString
        if (date()) {
            var dateTime = new Date(date());
            var hours = dateTime.getHours();
            // getMinutes returns the minutes in local time zone from 0 to 59
            var minutes = dateTime.getMinutes();
            var meridiem = " AM";

            // convert to 12-hour time format
            if (hours > 12) {
                hours = hours - 12;
                meridiem = ' PM'
            } else if (hours === 0) {
                hours = 12
            }

            // minutes should always be two digits long
            if (minutes < 10) {
                minutes = "0" + minutes.toString()
            }

            return hours + ':' + minutes + meridiem;
        } else {
            return "";
        }
    };
    
    /**
     * Returns the month numeric value from the provided date object.
     *
     * @param date The date object to fetch the month from
     */
    var _getDayNumber = function (date) {
        if (date()) {
            var cDate = new Date(date());
            var dayNum = cDate.toDateString().split(" ")[2];

            if (dayNum) {
                return dayNum;
            }
        } else {
            return "";
        }
    };

    /**
     * Returns the month char value from the provided date object.
     *
     * @param date The date object to fetch the month from
     */
    var _getMonthChar = function (date) {
        if (date()) {
            var cDate = new Date(date());
            var monthChar = cDate.toDateString().split(" ")[1];

            if (monthChar) {
                return monthChar;
            }
        } else {
            return "";
        }
    };

    /**
     * Returns the true if the URL is valid, else return false.
     *
     * @param url to validate
     */
    var _isValidUrl = function (url) {

        var pattern = new RegExp('^(https?:\\/\\/)?' + // protocol
               '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' + // domain name
               '((\\d{1,3}\\.){3}\\d{1,3}))' + // OR ip (v4) address
               '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + // port and path
               '(\\?[;&a-z\\d%_.~+=-]*)?' + // query string
               '(\\#[-a-z\\d_]*)?$', 'i'); // fragment locator

        return !!pattern.test(url);
    };

    return {
        formatMessage: formatMessage,
        getParameterByName: _getParameterByName,
        isWindow: _isWindow,
        isJSON: _isJSON,
        getTimeString: _getTimeString,
        getDayNumber: _getDayNumber,
        getMonthChar: _getMonthChar,
        isValidUrl: _isValidUrl
    }
})();

if (typeof define === "function" && define.amd) {
    define("utility", [], function () { return utility; });
}