﻿/**
 * Responsible for managing the kSocial integration on the social html template
 * fragment.
 * 
 * Author: Waleed Madanat
 */
var socialManager = (function () {
    var SOCIAL_DIV_ID = 'social_fragment';
    var SOCIAL_PLUGIN_CONTAINER = 'social_plugin_container';

    var socialContainer;

    

    var appModel;

    /**
     * Handles the updates to the kSocial integration based on the
     * observable SocialEnabled property of the ViewModel.
     *
     * This is used in conjunction with the frag_social.html
     */
    function _handleSocial(status, uniqueId) {
        var $template = $("#template");
        socialContainer = $('.kSocialContainer:first');
        if (socialContainer.length > 0) {
            socialContainer = socialContainer[0];
            if (!socialContainer.id)
                socialContainer.id = SOCIAL_DIV_ID;
        }
        else {
            socialContainer = document.createElement('div');
            socialContainer.id = SOCIAL_DIV_ID;
            socialContainer.className = 'kSocialContainer';
            $template.append(socialContainer);
        }

        require(['knockout', 'LocalModel'], function (ko, model) {
            appModel = model.appModel();
            
            var viewModel = ko.dataFor($template[0]);

            if (viewModel.Info && viewModel.Info.SocialEnabled) {
                var socialStatus = viewModel.Info.SocialEnabled() && status;
                if (!uniqueId)
                    uniqueId = viewModel.Info.InstanceId();

                // Load/Unload social if it is enabled
                _socialLoader(socialStatus, viewModel, uniqueId);

                // Subscribe to updates on socialEnabled
                viewModel.Info.SocialEnabled.subscribe(function (newSocialStatus) {
                    newSocialStatus = newSocialStatus && status;
                    _socialLoader(newSocialStatus, viewModel, uniqueId);
                });
            }
        });
    }

    /**
     * Private function to load/unload social integration.
     */
    function _socialLoader(status, viewModel, uniqueId) {
        // Show/Hide the social plugin
        socialContainer.style.display = (status ? 'block' : 'none');
        if (status) {
            //var likes = new kSocial.Likes(LIKES_DIV_ID, '52cee8d54975bb2022000001'/*appModel.SocialAppId*/, viewModel.Info.Name(), null, 'deviceId', device.platform);
            var deviceId = null;
            if (kPush && kPush.deviceId) {
                deviceId = kPush.deviceId;
            }
            var comments = new kSocial.Comments(SOCIAL_DIV_ID, appModel.SocialAppId, uniqueId, viewModel.Info.Name(), deviceId, device.platform, null, function (err, result) {

            });
        }
        else {
            $(socialContainer).hide();
            $('.ksocialCommentBox').remove();
        }
    }

    // Public
    return {
        handleSocial: _handleSocial
    }
})();

$(function () {
    if (typeof (getSocialUniqueId) === "function") {
        getSocialUniqueId(function (uniqueId) {
            var status = false;
            if (uniqueId) status = true;

            console.log('calling handleSocial with uniqueId item: ' + uniqueId);
            socialManager.handleSocial(status, uniqueId);
        });
    }
    else {
        console.log('calling handleSocial with InstanceId');
        socialManager.handleSocial(true, null);
    }
});