﻿// Depends on tmp_menu.js

window.currentPage = 0;

/**
 * Applies sliding animation for expandable fields.
 *
 * @param id The id of the field to expand (jQuery Id selector value)
 * @param imgId The id of the image showing up or down (jQuery Id selector value)
 * @param imgUrl The URL of the image to use for the intended state
 * @param height The height in pixels (numeric)
 */
function slide(id, imgId, imgUrl, height, callback) {
    var content = $(id);

    // Decide whether to allow overflow based on the height
    if (height <= 30) {
        content.css('overflow', 'hidden');
    } else {
        content.css('overflow', 'auto');
    }

    // Change icon and animate
    if (imgId && imgUrl) {
        $(imgId).css('background-image', imgUrl);
    }

    content.animate({ height: height + 'px' }, 300);

    if (callback) {
        callback();
    }
}

/**
 * Utility method navigate the back for single and multi paged widgets.
 * It relies on a currentPage property that needs to be updated by
 * the widgets.
 *
 * @param manager The WidgetsManager instance
 * @param thelper The TemplateHelper instance
 */
function navigateBack() {
    // Hide the menu if already open. From tmp_menu.js
    hideMenu();

    // In case of multi-page widgets, the widget JS file should contain
    // a previousPage function to navigate to the previous page
    // based on a global parameter currentPage
    if (!currentPage || currentPage <= 0) {
        require(['WidgetManager'], function (manager) {
            manager.navigateBack();
        });
    } else {
        // This method should be implemented in the scripts that have multiple pages
        previousPage();
    }
}

/**
 * Handle home button click on the menu bar.
 */
function goHome() {
    currentPage = 0;

    require(['TemplatesHelper', 'WidgetManager'], function (thelper, manager) {
        manager.show(0, thelper.TEMPLATE_KEYS.home);
    });
}

/**
 * Opens a URL inside an in app browser.
 * 
 * @param url The URL to open
 */
function openUrl(url) {
    var hWin = window.open(url, '_blank', 'location=yes,toolbar=yes,enableViewportScale=yes');
}

/**
 * Sets the visibility of an element by updating its display style.
 * 
 * @param id The id of the element
 * @param show True shows the element, false hides it
 */
function setVisible(id, show) {
    if (show) {
        document.getElementById(id).style.display = 'block';
    } else {
        document.getElementById(id).style.display = 'none';
    }
}

$(function () {
    // Attach a click handler to all widget icons
    $("#back").off('click');
    $("#back").click(function () {
        navigateBack();
    });

    // Navigate to the home page on back button clicks
    $(document).off('backbutton');
    $(document).on('backbutton', function () {
        // If the media player is visible, hide it
        if (kMedia.isShown()) {
            kMedia.hide();
        }
        // Else, execute the normal back button behaviour
        else {
            navigateBack();
        }
    });

    // Attach a click handler to the menu button
    $("#shortCutMenu").off('click');
    $("#shortCutMenu").click(function (event) {
        // We need to prevent propagation otherwise the template hook
        // will catch this event and reverse it
        event.stopPropagation();

        // From tmp_menu.js
        triggerMenu();
    });

    // Attach a click event to the template div that'll make the menu go away
    $("#template").off('click');
    $("#template").click(function (event) {
        // From tmp_menu.js
        hideMenu();
    });

    console.log("Live events registered!");
});